/****************************************************************************/
/* tiff.h    (c) Copyright Black Ice Software Inc.  1989 - 1991             */
/*                   All Rights Reserved                                    */
/*                   Unpublished and confidential material.                 */
/*                   --- Do not Reproduce or Disclose ---                   */
/****************************************************************************/
/***************************************************************************
 Module
      tiff.h

 Revision
      $Revision:   1.2  $

 Description
      Declaration of functions and macros used in TIFF file manipulation.

 Notes
      The function definitions are in TIFF.C .

 
   $Log:   E:/c/tiff/tiff.h_v  $
   
      Rev 1.2   26 Mar 1991  9:43:26   H_G
   Reference version. Supports multiple images and banding.
   
      Rev 1.0   24 Jan 1991 11:37:30   H_G
   Initial revision.
***************************************************************************/

/* ---------------------------- Module Defines --------------------------- */

// TIFF tag identifiers 
#define NEWSUBFILETYPE            254
#define SUBFILETYPE               255
#define IMAGEWIDTH                256 
#define IMAGELENGTH               257
#define BITSPERSAMPLE             258
#define COMPRESSION               259
#define PHOTOMETRICINTERPRETATION 262
#define THRESHHOLDING             263
#define CELLWIDTH                 264
#define CELLLENGTH                265
#define FILLORDER                 266
#define DOCUMENTNAME              269
#define IMAGEDESCRIPTION          270
#define MAKE                      271
#define MODEL                     272
#define STRIPOFFSETS              273
#define ORIENTATION               274
#define SAMPLESPERPIXEL           277
#define ROWSPERSTRIP              278
#define STRIPBYTECOUNTS           279
#define MINIMUMVALUE              280
#define MAXIMUMVALUE              281
#define XRESOLUTION               282
#define YRESOLUTION               283
#define PLANARCONFIG              284
#define PAGENAME                  285
#define XPOSITION                 286
#define YPOSITION                 287
#define FREEOFFSETS               288
#define FREEBYTECOUNTS            289
#define GRAYRESPONSEUNITS         290
#define GRAYRESPONSECURVE         291
#define GROUP3OPTIONS             292
#define GROUP4OPTIONS             293
#define RESOLUTIONUNIT            296
#define PAGENUMBER                297
#define COLORRESPONSEUNIT         300
#define COLORRESPONSECURVES       301
#define SOFTWARE                  305
#define DATETIME                  306
#define ARTIST                    315
#define HOSTCOMPUTER              316
#define PREDICTOR                 317
#define WHITEPOINT                318
#define PRIMARYDHROMATICITIES     319
#define COLORMAP                  320
#define EXISTINGTAGS              0   /* Special tag. See documentation. */


// Image types as defined in tag NEWSUBFILETYPE. 
#define REDUCEDVERSION    1
#define APAGEOFAMULTIPAGE 2
#define TRANSPARENCYMASK  4
// Extended image types used as filters with the above ones. 
#define NORMALVERSIONS    0
#define REGISTERED        (1<<14)
#define ALLVERSIONS       (1<<15)

// Image types as defined in tag SUBFILETYPE (not recommended any more).
#define OLDFULLVER        1
#define OLDREDUCEDVER     2
#define OLDAPAGE          3

// Compression types.
#define NOCOMPRESSION     1
#define TIFF_CCITT        2
#define CCITT_GROUP3      3
#define CCITT_GROUP4      4
#define LZW_COMPRESSION   5
#define PACK_BITS         32773
// Additional info on the compression algorithm. It can be OR-ed 
// to the DWORD compression value as the high word.
#define CCITT_1D         0
#define CCITT_2D         (1L << 16)
#define CCITT_UNCOMP     (2L << 16)
#define CCITT_FILLBITS   (4L << 16)
#define LZW_NORMAL       (1L << 16)
#define LZW_HORIZDIFF    (2L << 16)

// Photometric interpretation values.
#define PI_GREYSCALE       0  /* Greyscale or bilevel, 0 is white. */
#define PI_INV_GREYSCALE   1  /* Greyscale or bilevel, 0 is black. */
#define PI_RGBCOLOR        2  /* RGB image, (0,0,0) represents black. */
#define PI_PALETTECOLOR    3  /* Palette color image. See Colormap. */
#define PI_TRANSPMASK      4  /* Transparency mask. 1 defines visible plot. */

// Maximum length of a full file name. 
#define MAXPATH 150  
// Maximum number of images in a TIFF file.
#define MAXIMAGENUM 8 


// OpenTiffFile() nMode parameters. 
#define T_READ     READ          /* 0 */
#define T_WRITE    WRITE         /* 1  Not used with OpenTiffFile() */
#define T_APPEND   READ_WRITE    /* 2  Not used with OpenTiffFile() */
#define T_CREATE   3


// Interprets a number referenced by pointer 'Buffer' as WORD or DWORD 
// depending on the value of 'Type' (SHORTTYPE or LONGTYPE). The return
// value is always has type of DWORD.
#define SHORT_OR_LONG(Buffer,Type)   ( (Type==SHORTTYPE) ?   \
        (DWORD) ( *(WORD FAR *)Buffer ) : (*(DWORD FAR *)Buffer ) )

// As previous, but for the Index-th item in Buffer.
#define SHORT_OR_LONG_PTR(Buffer,Index,Type)   ( (Type==SHORTTYPE) ?   \
        (DWORD) (((WORD FAR *)Buffer)[Index]) : (((DWORD FAR *)Buffer)[Index]))

#define MAX_ERROR_LENGTH 60

/* ----------------------------- Module Types ---------------------------- */

typedef HANDLE TIFFFILE;    // TIFF image chain structure handle.


// TIFF RATIONALETYPE structure.
typedef struct tag_RATIO {
  DWORD    dwNumer;
  DWORD    dwDenom;
  }  RATIO;


/* TiffError() return values. */
enum tag_TIFFERR {
     TOK,                  // Operation was successfull.
     TNOTENOUGHMEMORY,     // Not enough memory to allocate block.
     TBADOPENMODE,         // Not T_READ or T_CREAT in OpenTiffFile().
     TINDEXOUTOFRANGE,     // Image or tag data index is out of range.
     TSTRIPOUTOFRANGE,     // Strip index is out of range.
     TENDOFDATA,           // Unexpected end of file in reading data.
     TFILEIOERROR,         // General I/O error in reading or writting.
     TPATHTOOLONG,         // Pathname is longer than MAXPATH.
     TNOTTIFFILE,          // File has not a valid TIFF format.
     TBADTAG,              // Invalid TIFF directory tag.
     TTOOFEWINFO,          // Indispensable tags are missing.
     TUNKNOWNCOMPRESSION,  // Image compression type is not known.
     TCOMPRESSIONFAILED,   // Error during compression or decompression.
     TNOSUCHTAG,           // Such tag is not defined.
     TBADBYTEORDER,        // Invalid byte order ID in ModiTiffFileInfo().
     TBADBITORDER,         // Invalid BITORDER tag value.
     TBADNUMBEROFCOLORS,   // Invalid BITSPERPIXEL tag value.
     TTOFEWCOLORINFO       // Color info is not enough.
                 } TIFFERR;

// TIFF image chain information structure (used with GetTiffFileInfo()).
typedef struct tag_TIFFHDR {
  char    cFileName[MAXPATH];     // Associated filename.
  int     nByteOrder;             // Byte order is revers (High/Low) ?
  int     iNImages;               // Number of images in the file.
  DWORD   dwFirstTID;             // Offset of the 1st TID in the file.
  } TIFFHDR;



/* --------------------------- Global Functions -------------------------- */

/* Functions manipulating on TIFF image chains */
TIFFFILE FAR PASCAL OpenTiffFile(LPSTR lpszFileName, int iMode);
BOOL     FAR PASCAL CloseTiffFile(TIFFFILE hChain);
BOOL     FAR PASCAL DropTiffImage(TIFFFILE hChain, int nImage);

/* Functions registering images and retrieving tag info. */
BOOL     FAR PASCAL GetTiffImage(TIFFFILE hChain, int iImage);
BOOL     FAR PASCAL GetTiffImageInfoBuffer(TIFFFILE hChain, int iImage, 
                                 WORD wTagID, HANDLE FAR *lphData,
                                 WORD FAR *lpwType, DWORD FAR *lpdwCount);
BOOL     FAR PASCAL GetTiffImageInfo(TIFFFILE hChain, int iImage,
                                     WORD wTagID, VOID FAR *lpData);
int      FAR PASCAL GetTiffDimensions(TIFFFILE hChain, int nImage, 
                                      LPBITMAPINFOHEADER lpBIH);
HANDLE   FAR PASCAL GetTiffRGBQuad(TIFFFILE hChain, int nImage);
WORD     FAR PASCAL GetTiffStripCount(TIFFFILE hChain, int nImage);
WORD     FAR PASCAL GetTiffRowsPerStrip(TIFFFILE hChain, int nImage);

/* Function decoding TIFF image. */
BOOL     FAR PASCAL DecodeTiffImage(TIFFFILE hChain, int nImage, 
                                    WORD wFirstStrip, WORD wNumLines,
                                    LONG lUserData, FARPROC lpLineFn);
BOOL FAR PASCAL DecodeCcittImage(int nFile, WORD wWidth, WORD wLength,
                                 DWORD dwCompression, LONG lUserData, 
                                 FARPROC lpLineFn);

/* Defining TIFF images and tags. */
int      FAR PASCAL DefineTiffImage(TIFFFILE hChain);
BOOL     FAR PASCAL DefineTiffImageInfoBuffer(TIFFFILE hChain, int nImage,
                                              WORD wTagID, VOID FAR *lpData, 
                                              WORD wType, DWORD dwCount);
BOOL     FAR PASCAL DefineTiffImageInfo(TIFFFILE hChain, int nImage,
                                        WORD wTagID, WORD wType, LONG lData);

/* Functions encoding images into TIFF. */
BOOL     FAR PASCAL EncodeTiffImage(TIFFFILE hChain, int nImage, LONG lUserData, FARPROC lpLineFn);
BOOL     FAR PASCAL EncodeCcittImage(int nFile, WORD wWidth, WORD wHeight,
                                     DWORD dwCompression, LONG lUserData, 
                                     FARPROC lpLineFn);
int      FAR PASCAL DefineGeneralTags(TIFFFILE hChain, int nImage, BITMAP Bm);

/* Testing and error functions. */
BOOL FAR PASCAL     ModiTiffFileInfo(TIFFFILE hChain, LPSTR lpszFileName,
                                     int nByteOrder);
BOOL     FAR PASCAL GetTiffFileInfo(TIFFFILE hChain, TIFFHDR FAR *Info);
BOOL     FAR PASCAL IsTiffImageRegistered(TIFFFILE hChain, int iImage);
int      FAR PASCAL NumberOfTiffImages(TIFFFILE hChain, WORD wImageType);
LPSTR    FAR PASCAL TiffErrorString(LPSTR lpszErrorStr, int nError);
int      FAR PASCAL TiffError( void );
int      FAR PASCAL SetTiffError(int nError);
   
//  High level functions to load tiff into bitmap and save bitmap in tiff 
HBITMAP  FAR PASCAL LoadTiffFileIntoBitmap(LPSTR lpszFileName, int nImage, BOOL bDispInfo);
HBITMAP  FAR PASCAL LoadTiffIntoBitmap(TIFFFILE hChain, int nImage);
BOOL     FAR PASCAL LoadTiffFileIntoClipboard(LPSTR lpszFileName, int nImage, BOOL bDispInfo);

int      FAR PASCAL TiffDllInfoDlgFn(HWND hDlg, WORD wMsg, WORD wParam, DWORD lParam);
int      FAR PASCAL DllFlushTiffLine(LPSTR lpLineBuff, int nLine, LONG lParam);
BOOL     FAR PASCAL SaveBitmapInTiffFile(LPSTR   lpszFileName, 
                                         HBITMAP hBitmap, 
                                         WORD    wCompMode, 
                                         BOOL    DisplayDlg);
BOOL     FAR PASCAL SaveBitmapInCcittFile(LPSTR   lpszFileName, 
                                          HBITMAP hBitmap, 
                                          WORD    wCompression);

BOOL     FAR PASCAL SaveTiffFileFromClipboard(LPSTR   lpszFileName, 
                                              WORD    wCompMode, 
                                              BOOL    DisplayDlg);


/* --------------------------- Global Variables -------------------------- */

/* ----------------------------- End of file ----------------------------- */
